<?php

namespace Survey\Survey\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Survey\Survey\Http\Requests\DistributionRequest;
use Survey\Survey\Interfaces\DistributionRepositoryInterface;
use Survey\Survey\Models\Distribution;

/**
 * Resource controller class for distribution.
 */
class DistributionResourceController extends BaseController
{

    /**
     * Initialize distribution resource controller.
     *
     * @param type DistributionRepositoryInterface $distribution
     *
     * @return null
     */
    public function __construct(DistributionRepositoryInterface $distribution)
    {
        parent::__construct();
        $this->repository = $distribution;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Survey\Survey\Repositories\Criteria\DistributionResourceCriteria::class);
    }

    /**
     * Display a list of distribution.
     *
     * @return Response
     */
    public function index(DistributionRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Survey\Survey\Repositories\Presenter\DistributionPresenter::class)
                ->$function();
        }

        $distributions = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('survey::distribution.names'))
            ->view('survey::distribution.index', true)
            ->data(compact('distributions', 'view'))
            ->output();
    }

    /**
     * Display distribution.
     *
     * @param Request $request
     * @param Model   $distribution
     *
     * @return Response
     */
    public function show(DistributionRequest $request, Distribution $distribution)
    {

        if ($distribution->exists) {
            $view = 'survey::distribution.show';
        } else {
            $view = 'survey::distribution.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('survey::distribution.name'))
            ->data(compact('distribution'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new distribution.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DistributionRequest $request)
    {

        $distribution = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('survey::distribution.name')) 
            ->view('survey::distribution.create', true) 
            ->data(compact('distribution'))
            ->output();
    }

    /**
     * Create new distribution.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DistributionRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $distribution                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('survey::distribution.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('survey/distribution/' . $distribution->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/survey/distribution'))
                ->redirect();
        }

    }

    /**
     * Show distribution for editing.
     *
     * @param Request $request
     * @param Model   $distribution
     *
     * @return Response
     */
    public function edit(DistributionRequest $request, Distribution $distribution)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('survey::distribution.name'))
            ->view('survey::distribution.edit', true)
            ->data(compact('distribution'))
            ->output();
    }

    /**
     * Update the distribution.
     *
     * @param Request $request
     * @param Model   $distribution
     *
     * @return Response
     */
    public function update(DistributionRequest $request, Distribution $distribution)
    {
        try {
            $attributes = $request->all();

            $distribution->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('survey::distribution.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('survey/distribution/' . $distribution->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('survey/distribution/' . $distribution->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the distribution.
     *
     * @param Model   $distribution
     *
     * @return Response
     */
    public function destroy(DistributionRequest $request, Distribution $distribution)
    {
        try {

            $distribution->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('survey::distribution.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('survey/distribution/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('survey/distribution/' . $distribution->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple distribution.
     *
     * @param Model   $distribution
     *
     * @return Response
     */
    public function delete(DistributionRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('survey::distribution.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('survey/distribution'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/survey/distribution'))
                ->redirect();
        }

    }

    /**
     * Restore deleted distributions.
     *
     * @param Model   $distribution
     *
     * @return Response
     */
    public function restore(DistributionRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('survey::distribution.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/survey/distribution'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/survey/distribution/'))
                ->redirect();
        }

    }

}
