<?php

namespace Survey\Survey\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Survey\Survey\Http\Requests\ReminderRequest;
use Survey\Survey\Interfaces\ReminderRepositoryInterface;
use Survey\Survey\Models\Reminder;
use Survey\Survey\Forms\Reminder as Form;

/**
 * APIController  class for reminder.
 */
class ReminderAPIController extends BaseController
{

    /**
     * Initialize reminder resource controller.
     *
     * @param type ReminderRepositoryInterface $reminder
     *
     * @return null
     */
    public function __construct(ReminderRepositoryInterface $reminder)
    {
        parent::__construct();
        $this->repository = $reminder;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Survey\Survey\Repositories\Criteria\ReminderResourceCriteria::class);
    }

    /**
     * Display a list of reminder.
     *
     * @return Response
     */
    public function index(ReminderRequest $request)
    {
        return $this->repository
            ->setPresenter(\Survey\Survey\Repositories\Presenter\ReminderPresenter::class)
            ->paginate();
    }

    /**
     * Display reminder.
     *
     * @param Request $request
     * @param Model   $reminder
     *
     * @return Response
     */
    public function show(ReminderRequest $request, Reminder $reminder)
    {
        return $reminder->setPresenter(\Survey\Survey\Repositories\Presenter\ReminderListPresenter::class);
        ;
    }

    /**
     * Create new reminder.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ReminderRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('survey::reminder.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('survey/reminder/' . $reminder->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('survey/reminder');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the reminder.
     *
     * @param Request $request
     * @param Model   $reminder
     *
     * @return Response
     */
    public function update(ReminderRequest $request, Reminder $reminder)
    {
        try {
            $data = $request->all();

            $reminder->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('survey::reminder.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('survey/reminder/' . $reminder->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('survey/reminder/' . $reminder->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the reminder.
     *
     * @param Model   $reminder
     *
     * @return Response
     */
    public function destroy(ReminderRequest $request, Reminder $reminder)
    {
        try {
            $reminder->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('survey::reminder.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('survey/reminder/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('survey/reminder/' . $reminder->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
