<?php

namespace Survey\Survey\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Survey\Survey\Http\Requests\ReminderRequest;
use Survey\Survey\Interfaces\ReminderRepositoryInterface;
use Survey\Survey\Models\Reminder;

/**
 * Resource controller class for reminder.
 */
class ReminderResourceController extends BaseController
{

    /**
     * Initialize reminder resource controller.
     *
     * @param type ReminderRepositoryInterface $reminder
     *
     * @return null
     */
    public function __construct(ReminderRepositoryInterface $reminder)
    {
        parent::__construct();
        $this->repository = $reminder;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Survey\Survey\Repositories\Criteria\ReminderResourceCriteria::class);
    }

    /**
     * Display a list of reminder.
     *
     * @return Response
     */
    public function index(ReminderRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Survey\Survey\Repositories\Presenter\ReminderPresenter::class)
                ->$function();
        }

        $reminders = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('survey::reminder.names'))
            ->view('survey::reminder.index', true)
            ->data(compact('reminders', 'view'))
            ->output();
    }

    /**
     * Display reminder.
     *
     * @param Request $request
     * @param Model   $reminder
     *
     * @return Response
     */
    public function show(ReminderRequest $request, Reminder $reminder)
    {

        if ($reminder->exists) {
            $view = 'survey::reminder.show';
        } else {
            $view = 'survey::reminder.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('survey::reminder.name'))
            ->data(compact('reminder'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new reminder.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ReminderRequest $request)
    {

        $reminder = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('survey::reminder.name')) 
            ->view('survey::reminder.create', true) 
            ->data(compact('reminder'))
            ->output();
    }

    /**
     * Create new reminder.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ReminderRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $reminder                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('survey::reminder.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('survey/reminder/' . $reminder->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/survey/reminder'))
                ->redirect();
        }

    }

    /**
     * Show reminder for editing.
     *
     * @param Request $request
     * @param Model   $reminder
     *
     * @return Response
     */
    public function edit(ReminderRequest $request, Reminder $reminder)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('survey::reminder.name'))
            ->view('survey::reminder.edit', true)
            ->data(compact('reminder'))
            ->output();
    }

    /**
     * Update the reminder.
     *
     * @param Request $request
     * @param Model   $reminder
     *
     * @return Response
     */
    public function update(ReminderRequest $request, Reminder $reminder)
    {
        try {
            $attributes = $request->all();

            $reminder->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('survey::reminder.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('survey/reminder/' . $reminder->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('survey/reminder/' . $reminder->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the reminder.
     *
     * @param Model   $reminder
     *
     * @return Response
     */
    public function destroy(ReminderRequest $request, Reminder $reminder)
    {
        try {

            $reminder->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('survey::reminder.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('survey/reminder/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('survey/reminder/' . $reminder->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple reminder.
     *
     * @param Model   $reminder
     *
     * @return Response
     */
    public function delete(ReminderRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('survey::reminder.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('survey/reminder'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/survey/reminder'))
                ->redirect();
        }

    }

    /**
     * Restore deleted reminders.
     *
     * @param Model   $reminder
     *
     * @return Response
     */
    public function restore(ReminderRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('survey::reminder.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/survey/reminder'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/survey/reminder/'))
                ->redirect();
        }

    }

}
