<?php

namespace Survey\Survey\Http\Controllers;

use App\Http\Controllers\APIController as BaseController;
use Survey\Survey\Http\Requests\ReportingRequest;
use Survey\Survey\Interfaces\ReportingRepositoryInterface;
use Survey\Survey\Models\Reporting;
use Survey\Survey\Forms\Reporting as Form;

/**
 * APIController  class for reporting.
 */
class ReportingAPIController extends BaseController
{

    /**
     * Initialize reporting resource controller.
     *
     * @param type ReportingRepositoryInterface $reporting
     *
     * @return null
     */
    public function __construct(ReportingRepositoryInterface $reporting)
    {
        parent::__construct();
        $this->repository = $reporting;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Survey\Survey\Repositories\Criteria\ReportingResourceCriteria::class);
    }

    /**
     * Display a list of reporting.
     *
     * @return Response
     */
    public function index(ReportingRequest $request)
    {
        return $this->repository
            ->setPresenter(\Survey\Survey\Repositories\Presenter\ReportingPresenter::class)
            ->paginate();
    }

    /**
     * Display reporting.
     *
     * @param Request $request
     * @param Model   $reporting
     *
     * @return Response
     */
    public function show(ReportingRequest $request, Reporting $reporting)
    {
        return $reporting->setPresenter(\Survey\Survey\Repositories\Presenter\ReportingListPresenter::class);
        ;
    }

    /**
     * Create new reporting.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ReportingRequest $request)
    {
        try {
            $data              = $request->all();
            $data['user_id']   = user_id();
            $data['user_type'] = user_type();
            $data              = $this->repository->create($data);
            $message           = trans('messages.success.created', ['Module' => trans('survey::reporting.name')]);
            $code              = 204;
            $status            = 'success';
            $url               = guard_url('survey/reporting/' . $reporting->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('survey/reporting');
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Update the reporting.
     *
     * @param Request $request
     * @param Model   $reporting
     *
     * @return Response
     */
    public function update(ReportingRequest $request, Reporting $reporting)
    {
        try {
            $data = $request->all();

            $reporting->update($data);
            $message = trans('messages.success.updated', ['Module' => trans('survey::reporting.name')]);
            $code    = 204;
            $status  = 'success';
            $url     = guard_url('survey/reporting/' . $reporting->getRouteKey());
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('survey/reporting/' . $reporting->getRouteKey());
        }
        return compact('data', 'message', 'code', 'status', 'url');
    }

    /**
     * Remove the reporting.
     *
     * @param Model   $reporting
     *
     * @return Response
     */
    public function destroy(ReportingRequest $request, Reporting $reporting)
    {
        try {
            $reporting->delete();
            $message = trans('messages.success.deleted', ['Module' => trans('survey::reporting.name')]);
            $code    = 202;
            $status  = 'success';
            $url     = guard_url('survey/reporting/0');
        } catch (Exception $e) {
            $message = $e->getMessage();
            $code    = 400;
            $status  = 'error';
            $url     = guard_url('survey/reporting/' . $reporting->getRouteKey());
        }
        return compact('message', 'code', 'status', 'url');
    }

    /**
     * Return the form elements as json.
     *
     * @param String   $element
     *
     * @return json
     */
    public function form($element = 'fields')
    {
        $form = new Form();
        return $form->form($element, true);
    }

}
