<?php

namespace Survey\Survey\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Survey\Survey\Http\Requests\ReportingRequest;
use Survey\Survey\Interfaces\ReportingRepositoryInterface;
use Survey\Survey\Models\Reporting;

/**
 * Resource controller class for reporting.
 */
class ReportingResourceController extends BaseController
{

    /**
     * Initialize reporting resource controller.
     *
     * @param type ReportingRepositoryInterface $reporting
     *
     * @return null
     */
    public function __construct(ReportingRepositoryInterface $reporting)
    {
        parent::__construct();
        $this->repository = $reporting;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Survey\Survey\Repositories\Criteria\ReportingResourceCriteria::class);
    }

    /**
     * Display a list of reporting.
     *
     * @return Response
     */
    public function index(ReportingRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Survey\Survey\Repositories\Presenter\ReportingPresenter::class)
                ->$function();
        }

        $reportings = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('survey::reporting.names'))
            ->view('survey::reporting.index', true)
            ->data(compact('reportings', 'view'))
            ->output();
    }

    /**
     * Display reporting.
     *
     * @param Request $request
     * @param Model   $reporting
     *
     * @return Response
     */
    public function show(ReportingRequest $request, Reporting $reporting)
    {

        if ($reporting->exists) {
            $view = 'survey::reporting.show';
        } else {
            $view = 'survey::reporting.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('survey::reporting.name'))
            ->data(compact('reporting'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new reporting.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ReportingRequest $request)
    {

        $reporting = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('survey::reporting.name')) 
            ->view('survey::reporting.create', true) 
            ->data(compact('reporting'))
            ->output();
    }

    /**
     * Create new reporting.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ReportingRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $reporting                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('survey::reporting.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('survey/reporting/' . $reporting->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/survey/reporting'))
                ->redirect();
        }

    }

    /**
     * Show reporting for editing.
     *
     * @param Request $request
     * @param Model   $reporting
     *
     * @return Response
     */
    public function edit(ReportingRequest $request, Reporting $reporting)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('survey::reporting.name'))
            ->view('survey::reporting.edit', true)
            ->data(compact('reporting'))
            ->output();
    }

    /**
     * Update the reporting.
     *
     * @param Request $request
     * @param Model   $reporting
     *
     * @return Response
     */
    public function update(ReportingRequest $request, Reporting $reporting)
    {
        try {
            $attributes = $request->all();

            $reporting->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('survey::reporting.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('survey/reporting/' . $reporting->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('survey/reporting/' . $reporting->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the reporting.
     *
     * @param Model   $reporting
     *
     * @return Response
     */
    public function destroy(ReportingRequest $request, Reporting $reporting)
    {
        try {

            $reporting->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('survey::reporting.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('survey/reporting/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('survey/reporting/' . $reporting->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple reporting.
     *
     * @param Model   $reporting
     *
     * @return Response
     */
    public function delete(ReportingRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('survey::reporting.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('survey/reporting'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/survey/reporting'))
                ->redirect();
        }

    }

    /**
     * Restore deleted reportings.
     *
     * @param Model   $reporting
     *
     * @return Response
     */
    public function restore(ReportingRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('survey::reporting.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/survey/reporting'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/survey/reporting/'))
                ->redirect();
        }

    }

}
